Package: midasHLA
Title: R package for immunogenomics data handling and association
        analysis
Version: 1.21.0
Authors@R: c(
  person("Christian", "Hammer", email = "hammerc6@gene.com", role = "aut"),
  person("Maciej", "Migdał", email = "mcjmigdal@gmail.com", role = c("aut", "cre")))
Description: MiDAS is a R package for immunogenetics data transformation and statistical analysis. MiDAS accepts input data in the form of HLA alleles and KIR types, and can transform it into biologically meaningful variables, enabling HLA amino acid fine mapping, analyses of HLA evolutionary divergence, KIR gene presence, as well as validated HLA-KIR interactions. Further, it allows comprehensive statistical association analysis workflows with phenotypes of diverse measurement scales. MiDAS closes a gap between the inference of immunogenetic variation and its efficient utilization to make relevant discoveries related to T cell, Natural Killer cell, and disease biology.
License: MIT + file LICENCE
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1), MultiAssayExperiment (>= 1.8.3)
Imports: assertthat (>= 0.2.0), broom (>= 0.5.1), dplyr (>= 0.8.0.1),
        formattable (>= 0.2.0.1), HardyWeinberg (>= 1.6.3), kableExtra
        (>= 1.1.0), knitr (>= 1.21), magrittr (>= 1.5), methods,
        stringi (>= 1.2.4), rlang (>= 0.3.1), S4Vectors (>= 0.20.1),
        stats, SummarizedExperiment (>= 1.12.0), tibble (>= 2.0.1),
        utils, qdapTools (>= 1.3.3)
Suggests: broom.mixed (>= 0.2.4), cowplot (>= 1.0.0), devtools (>=
        2.0.1), ggplot2 (>= 3.1.0), ggpubr (>= 0.2.5), rmarkdown,
        seqinr (>= 3.4-5), survival (>= 2.43-3), testthat (>= 2.0.1),
        tidyr (>= 1.1.2)
RoxygenNote: 7.1.1
VignetteBuilder: knitr
Collate: 'asserts.R' 'class.R' 'data.R' 'global.R' 'midasHLA.R'
        'parsingFunctions.R' 'stats.R' 'summarise.R'
        'transformationFunctions.R' 'utils.R'
biocViews: CellBiology, Genetics, StatisticalMethod
git_url: https://git.bioconductor.org/packages/midasHLA
git_branch: devel
git_last_commit: 3a3e981
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:48:23 UTC; biocbuild
Author: Christian Hammer [aut],
  Maciej Migdał [aut, cre]
Maintainer: Maciej Migdał <mcjmigdal@gmail.com>
