% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{listMiDASDictionaries}
\alias{listMiDASDictionaries}
\title{List HLA alleles dictionaries}
\usage{
listMiDASDictionaries(pattern = "allele", file.names = FALSE)
}
\arguments{
\item{pattern}{String used to match dictionary names, it can be a regular
expression. By default all names are matched.}

\item{file.names}{Logical value. If FALSE, only the names of dictionaries are
returned. If TRUE their paths are returned.}
}
\value{
Character vector giving names of available HLA alleles dictionaries.
}
\description{
\code{listMiDASDictionaries} lists dictionaries shipped with the MiDAS package.
See \code{\link{hlaToVariable}} for more details on dictionaries.
}
