% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class.R
\name{prepareMiDAS_hla_alleles}
\alias{prepareMiDAS_hla_alleles}
\title{Prepare MiDAS data on HLA allele level}
\usage{
prepareMiDAS_hla_alleles(hla_calls, ...)
}
\arguments{
\item{hla_calls}{HLA calls data frame, as returned by
\code{\link{readHlaCalls}} function.}

\item{...}{Not used}
}
\value{
Matrix
}
\description{
Prepare MiDAS data on HLA allele level
}
